from setuptools import setup, find_packages
import os

version = "0.0.0"
try:
    # import the package to get the version
    from wael_lib import __version__ as version
except Exception:
    pass

# Read the README for long description
this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="wael-lib",
    version=version,
    description="Technical Analysis, Data Handling & Sequence Utilities for Trading Applications",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Wael Fouda",
    url="https://github.com/WaelFouda/wael-lib",
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    python_requires=">=3.10",
    install_requires=[
        "pandas>=2.0.0",
        "numpy>=1.24.0",
        "scikit-learn>=1.3.0",
    ],
    license="MIT",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Financial and Insurance Industry",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Office/Business :: Financial",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
)
